#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import print_function
import argparse
import getpass
import os
import sys

from monolithe import MonolitheConfig
from monolithe.generators import SDKGenerator


def main(argv=sys.argv):
    """

    """
    parser = argparse.ArgumentParser(description="Generates a SDK according from a specification set")

    parser.add_argument("-g", "--github",
                        dest="api_url",
                        metavar="github_api_url",
                        help="The GitHub API URL. Can be given by setting the environment variable \"MONOLITHE_GITHUB_API_URL\"",
                        type=str)

    parser.add_argument("-l", "--login",
                        dest="login",
                        metavar="login_login",
                        help="The GitHub Login (if set, you will be prompted for your password). Can be given by setting the environment variable \"MONOLITHE_GITHUB_LOGIN\"",
                        type=str)

    parser.add_argument("-t", "--token",
                        dest="token",
                        metavar="github_token",
                        help="The GitHub Token (if set, --login will be ignored). To generate a token, go here https://github.com/settings/tokens. Can be given by setting the environment variable \"$MONOLITHE_GITHUB_TOKEN\"",
                        type=str)

    parser.add_argument("-o", "--organization",
                        dest="organization",
                        metavar="github_organization",
                        help="The GitHub Organization. Can be given by setting the environment variable \"MONOLITHE_GITHUB_ORGANIZATION\"",
                        type=str)

    parser.add_argument("-r", "--repository",
                        dest="repository",
                        metavar="github_repository",
                        help="The GitHub Repository. Can be given by setting the environment variable \"MONOLITHE_GITHUB_REPOSITORY\"",
                        type=str)

    parser.add_argument("-b", "--branches",
                        dest="branches",
                        metavar="branches",
                        help="The branches of the specifications to use to generate the documentation (examples: \"master 3.2\")",
                        nargs="*",
                        type=str)

    parser.add_argument("-p", "--path",
                        dest="repository_path",
                        metavar="path",
                        help="The relative repository path of the specification folder. Can be given by setting the environment variable \"MONOLITHE_GITHUB_REPOSITORY_PATH\"",
                        type=str)

    parser.add_argument("-f", "--folder",
                        dest="folder",
                        metavar="folder",
                        help="Path of the specifications folder. If set, all other attributes will be ignored",
                        type=str)

    parser.add_argument("-c", "--config",
                        dest="config_path",
                        metavar="config_path",
                        help="Path the monolithe configuration file",
                        type=str)

    parser.add_argument("-d", "--doc",
                        dest="generate_doc",
                        help="generate documentation of the SDK",
                        action="store_true")

    parser.add_argument("--vanilla-prefix",
                        dest="vanilla_prefix",
                        help="Prefix added to all vanilla path declared in the monolithe configuration file",
                        required=False,
                        type=str)

    parser.add_argument("--generation-version",
                        dest="generation_version",
                        help="Overwrite the sdk version given in monolithe.conf",
                        required=False,
                        type=str)

    parser.add_argument("-L", "--language",
                        dest="language",
                        help="Choose the output language of the SDK. Default is python",
                        default='python',
                        type=str)

    args = parser.parse_args()

    monolithe_config = None
    if args.config_path:
        monolithe_config = MonolitheConfig.config_with_path(args.config_path)

    if monolithe_config and args.vanilla_prefix:
        monolithe_config.set_option("user_vanilla", "%s/%s" % (args.vanilla_prefix, monolithe_config.get_option("user_vanilla", "transformer")), "transformer")

    if monolithe_config and args.generation_version:
        monolithe_config.set_option("version", args.generation_version, "transformer")

    if monolithe_config:
        monolithe_config.language = args.language

    generator = SDKGenerator(monolithe_config=monolithe_config)

    if args.folder:
        generator.initialize_folder_manager(folder=args.folder)
        if not monolithe_config:
            monolithe_config = generator.retrieve_monolithe_config_from_folder(language=args.language)
        generator.generate_from_folder()

    else:
        if not args.branches:
            print("You must provide the --branches options. Use --help for help.")
            sys.exit(1)

        # Use environment variable if necessary
        if not args.api_url and "MONOLITHE_GITHUB_API_URL" in os.environ:
            args.api_url = os.environ["MONOLITHE_GITHUB_API_URL"]

        if not args.token and not args.login:

            if "MONOLITHE_GITHUB_TOKEN" in os.environ:
                args.token = os.environ["MONOLITHE_GITHUB_TOKEN"]

            elif "MONOLITHE_GITHUB_LOGIN" in os.environ:
                args.login = os.environ["MONOLITHE_GITHUB_LOGIN"]

        if not args.organization and "MONOLITHE_GITHUB_ORGANIZATION" in os.environ:
            args.organization = os.environ["MONOLITHE_GITHUB_ORGANIZATION"]

        if not args.repository and "MONOLITHE_GITHUB_REPOSITORY" in os.environ:
            args.repository = os.environ["MONOLITHE_GITHUB_REPOSITORY"]

        if not args.repository_path and "MONOLITHE_GITHUB_REPOSITORY_PATH" in os.environ:
            args.repository_path = os.environ["MONOLITHE_GITHUB_REPOSITORY_PATH"]

        if not args.config_path and "MONOLITHE_CONFIG_PATH" in os.environ:
            args.config_path = os.environ["MONOLITHE_CONFIG_PATH"]

        if not args.repository_path:
            args.repository_path = "/"

        login_or_token = None
        password = None
        if args.token:
            password = None
            login_or_token = args.token
        elif args.login:
            login_or_token = args.login
            password = getpass.getpass(prompt="Enter your GitHub password for %s: " % args.login)

        generator.initialize_repository_manager(api_url=args.api_url,
                                                login_or_token=login_or_token,
                                                password=password,
                                                organization=args.organization,
                                                repository=args.repository,
                                                repository_path=args.repository_path)
        if not monolithe_config:
            monolithe_config = generator.retrieve_monolithe_config_from_repo(branch=args.branches[0], language=args.language)

        generator.generate_from_repo(branches=args.branches)

    # Generate SDK documentation
    if args.generate_doc:
        generator.generate_documentation()

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals

from future import standard_library
standard_library.install_aliases()
from builtins import object

import base64
import json
import os
import tempfile
import requests
import configparser
from io import StringIO
import zipfile

from github import Github, InputGitTreeElement
from monolithe.lib import apply_extension

from .specification import Specification

MODE_NORMAL = 1
MODE_RAW_SPECS = 2
MODE_RAW_ABSTRACTS = 3


class RepositoryManager (object):
    """ RepositoryManager is an object that allows to manipulate the API specification repository
    """

    def __init__(self, monolithe_config, api_url, login_or_token, password, organization, repository, repository_path="/"):
        """ Initialize RepositoryManager

            Args:
                api_url: the API url for GitHub
                login_or_token: the authentication token or login for GitHub
                password: the authentication password for GitHub (only if login_or_token is a username)
                organization: the organization where specifications_repository is
                repository: the repository containing the specifications
        """
        self.monolithe_config = monolithe_config
        self._organization = organization
        self._repository = repository
        self._repository_path = repository_path

        if self._repository_path[0] == "/":
            self._repository_path = self._repository_path[1:]

        if len(self._repository_path) > 1:
            if self._repository_path[-1] == "/":
                self._repository_path = self._repository_path[:-1]

        self._github = Github(login_or_token=login_or_token, password=password, base_url=api_url)
        self._repo = self._github.get_repo("%s/%s" % (organization, repository))

    @property
    def organization(self):
        return self._organization

    @property
    def repository(self):
        return self._repository

    @property
    def repository_path(self):
        return self._repository_path

    def get_available_branches(self):
        """ Returns the list of available API spec branches

            Returns:
                list of all available specification branches
        """

        return [branch.name for branch in self._repo.get_branches()]

    def get_available_specifications(self, branch="master"):
        """ Returns the list of available specification files

            Args:
                branch: the branch where to find files (default: "master")

            Returns:
                list of all available specification files in the given branch
        """

        ret = []

        for file in self._repo.get_dir_contents(self._repository_path, ref=branch):

            if os.path.splitext(file.name)[1] != ".spec" or file.name.startswith("@"):
                continue

            ret.append(file.name)

        return ret

    def get_api_info(self, branch="master"):
        """
            Returns the content of the api.info in the specification

            Args:
                branch: the branch where to the api info (default: "master")

            Returns:
                the server api version as dict containing the keys "version", "prefix" and "root"
        """
        path = os.path.normpath("%s/%s" % (self._repository_path, "api.info"))
        try:
            return json.loads(base64.b64decode(self._repo.get_file_contents(path, ref=branch).content))
        except Exception as e:
            raise Exception("could not parse api.info", e)

    def get_monolithe_config(self, branch="master"):
        """
            Returns the content of the monolithe config

            Args:
                branch: the branch where to the monolithe config (default: "master")

            Returns:
                the ConfigParserObject
        """
        path = os.path.normpath("%s/%s" % (self._repository_path, "monolithe.ini"))
        try:
            data = base64.b64decode(self._repo.get_file_contents(path, ref=branch).content)
            string_buffer = StringIO.StringIO(data)
            monolithe_config_parser = configparser.ConfigParser()
            monolithe_config_parser.readfp(string_buffer)
            return monolithe_config_parser

        except Exception as e:
            raise Exception("could not parse monolithe.ini", e)

    def get_last_commit(self, branch="master"):
        """
            Returns the last commit in the given branch

            Args:
                branch: the branch where to the commit (default: "master")
        """

        return self._repo.get_commits()[0]

    def get_repository_push_permission(self):
        """
            Returns if current user can push to the current repository
        """
        return self._repo.permissions.push

    def get_upstream_repository(self):
        """
            Returns information of the upstream repository
            if it exists, otherwise, None
        """
        return self._repo.parent

    def get_all_specifications(self, branch="master", mode=MODE_NORMAL):
        """ Returns all availables specifications using zipball feature of GitHub
            This is extremely fast if you need to get a lot of Specifications in one
            shot.

            Args:
                branch: the branch where to find files (default: "master")

            Returns:
                list of Specification objects.
        """
        specifications = {}
        archive_fd, archive_path = tempfile.mkstemp("archive.zip")
        url = self._repo.get_archive_link("zipball", ref=branch)
        req = requests.get(url, stream=True, verify=False)

        # retrieve and write the archive content to a temporary file
        with open(archive_path, "wb") as f:
            for chunk in req.iter_content(chunk_size=1024):
                if not chunk:
                    continue
                f.write(chunk)
                f.flush()

        stripped_repository_path = self.repository_path.strip("/")

        # reads the content of the archive and generate Specification objects
        with zipfile.ZipFile(archive_path, "r") as archive_content:
            for file_name in archive_content.namelist():

                spec_dir = os.path.split(file_name)[0]

                if not spec_dir.startswith(stripped_repository_path):
                    continue

                spec_name = os.path.split(file_name)[1]

                if spec_name == "api.info":
                    continue

                ext = os.path.splitext(spec_name)[1]
                is_abstract = spec_name.startswith("@")

                if ext != ".spec":
                    continue

                if mode in (MODE_NORMAL, MODE_RAW_SPECS) and is_abstract:
                    continue

                if mode == MODE_RAW_ABSTRACTS and not is_abstract:
                    continue

                specifications[spec_name.replace(".spec", "")] = Specification(filename=spec_name, data=self.get_specification_data(name=spec_name, archive=archive_content, mode=mode), monolithe_config=self.monolithe_config)

        # cleanup the temporary archive
        os.close(archive_fd)
        os.remove(archive_path)

        return specifications

    def get_specification_data(self, name, branch="master", archive=None, mode=MODE_NORMAL):
        """ Returns the content of the specification_file in the given branch

            Args:
                name: the name of the specification file of which you want to get the content
                branch: the branch where to find files (default: "master")

            Returns:
                JSON decoded structure of the specification file.
        """
        data = {}
        full_path = None

        if archive:
            full_path = os.path.normpath("%s%s/%s" % (archive.namelist()[0], self._repository_path, name))
            try:
                data = json.loads(archive.read(full_path))
            except Exception as e:
                raise Exception("could not parse %s" % name, e)
        else:
            full_path = os.path.normpath("%s/%s" % (self._repository_path, name))
            try:
                data = json.loads(base64.b64decode(self._repo.get_file_contents(full_path, ref=branch).content))
            except Exception as e:
                raise Exception("could not parse %s" % name, e)

        if mode == MODE_NORMAL and "model" in data and "extends" in data["model"]:
            for extension in data["model"]["extends"]:
                apply_extension(self.get_specification_data(name="%s.spec" % extension, branch=branch, archive=archive, mode=MODE_NORMAL), data)

        return data

    def get_specification(self, name, branch="master", archive=None, mode=MODE_NORMAL):
        """ Returns a Specification object from the given specification file name in the given branch

            Args:
                name: the name of the specification file of which you want to get the content
                branch: the branch where to find files (default: "master")

            Returns:
                Specification object.
        """
        return Specification(filename=name, data=self.get_specification_data(name, branch, archive), monolithe_config=self.monolithe_config)

    def save_specification(self, specification, message, branch="master"):
        """
        """
        self._commit(filename=specification.filename,
                     content=json.dumps(specification.to_dict(), indent=4, sort_keys=True),
                     message=message,
                     branch=branch)

    def rename_specification(self, specification, old_name, message, branch="master"):
        """
        """
        self._delete(filename=old_name, message=message, branch=branch)
        self.save_specification(specification=specification, message=message, branch=branch)

    def delete_specification(self, specification, message, branch):
        """
        """
        self._delete(filename=specification.filename, message=message, branch=branch)

    def save_apiinfo(self, version, root_api, prefix, message, branch="master"):
        """
        """
        self._commit(filename='api.info',
                     content=json.dumps({"prefix": prefix, "root": root_api, "version": version}, indent=4, sort_keys=True),
                     message=message,
                     branch=branch)

    def save_monolithe_config(self, monolithe_config_parser, message, branch="master"):
        """
        """
        string_buffer = StringIO.StringIO()
        monolithe_config_parser.write(string_buffer)
        data = string_buffer.getvalue()

        self._commit(filename='monolithe.ini',
                     content=data,
                     message=message,
                     branch=branch)

    def merge_upstream_master(self, local_branch, upstream_branch="master", commit_message="auto merge from monolithe"):
        """
            Synchronizes current parent repo's given branch into given local branch

            Note:
                If the local repository doesn't have an upstream, this function will return False

            Args:
                local_branch: the name of the branch of the local repository you want to merge into
                upstream_repo: the name of the upstream repository branch you want to merge
                commit_message: the message for the merge commit

            Returns:
                True in case of success, or False if the local repo doesn't have an upstream
        """
        upstream_repo = self.get_upstream_repository()

        if not upstream_repo:
            return False

        upstream_last_commit = upstream_repo.get_branch(upstream_branch).commit.sha
        self._repo.merge(base=local_branch, head=upstream_last_commit, commit_message=commit_message)

        return True

    def _delete(self, filename, message, branch):
        """
        """
        # ugly manual porting of https://github.com/PyGithub/PyGithub/pull/316/files
        path = os.path.join(self._repository_path, filename)
        sha = self._repo.get_file_contents(path, branch).sha
        parameters = {"message": message, "sha": sha, "branch": branch}

        self._repo._requester.requestJsonAndCheck("DELETE", self._repo.url + "/contents/" + path, input=parameters)

    def _commit(self, filename, content, message, branch, remove_trailing_whitespaces=True):
        """
        """
        head_ref = self._repo.get_git_ref("heads/%s" % branch)
        latest_commit = self._repo.get_git_commit(head_ref.object.sha)
        base_tree = latest_commit.tree

        if remove_trailing_whitespaces:
            content = '\n'.join([line.rstrip() for line in content.split('\n')])

        new_tree = self._repo.create_git_tree(
            [InputGitTreeElement(
                path="%s" % os.path.join(self._repository_path, filename),
                mode="100644",
                type="blob",
                content=content
            )],
            base_tree)

        new_commit = self._repo.create_git_commit(
            message=message,
            parents=[latest_commit],
            tree=new_tree)

        head_ref.edit(sha=new_commit.sha, force=False)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals
from future import standard_library
standard_library.install_aliases()
from builtins import object

import json
import os
import configparser

from .specification import Specification
from monolithe.lib import apply_extension


class FolderManager (object):
    """ RepositoryManager is an object that allows to manipulate the API specification repository
    """

    def __init__(self, folder, monolithe_config):
        """
        """
        self.monolithe_config = monolithe_config
        self._folder = folder

    @property
    def folder(self):
        return self._folder

    def get_available_specifications(self):
        """ Returns the list of available specification files

            Args:
                branch: the branch where to find files (default: "master")

            Returns:
                list of all available specification files in the given branch
        """
        ret = []
        for filename in os.listdir(self._folder):
            if os.path.splitext(filename)[1] != ".spec" or filename.startswith("@"):
                continue
            ret.append(filename)
        return ret

    def get_api_info(self):
        """
        """
        with open("%s/api.info" % self._folder, "r") as f:
            try:
                return json.loads(f.read())
            except Exception as e:
                raise Exception("could not parse api.info", e)

    def get_monolithe_config(self, branch="master"):
        """
        """
        with open("%s/monolithe.ini" % self._folder, "r") as f:
            try:
                monolithe_config_parser = configparser.ConfigParser()
                monolithe_config_parser.readfp(f)
                return monolithe_config_parser

            except Exception as e:
                raise Exception("could not parse monolithe.ini", e)

    def get_all_specifications(self):
        """
        """
        specifications = {}
        for name in self.get_available_specifications():
            specifications[name.replace(".spec", "")] = self.get_specification(name)
        return specifications

    def get_specification_data(self, name):
        """
        """
        data = {}
        with open("%s/%s" % (self._folder, name), "r") as f:
            try:
                data = json.loads(f.read())
                if "model" in data and "extends" in data["model"]:
                    for extension in data["model"]["extends"]:
                        apply_extension(self.get_specification_data(name="%s.spec" % extension), data)
            except Exception as e:
                raise Exception("Could not parse %s" % name, e)
        return data

    def get_specification(self, name):
        """
        """
        return Specification(filename=name, data=self.get_specification_data(name), monolithe_config=self.monolithe_config)

    def get_specifications(self, names, callback=None):
        """
        """
        specifications = []
        for name in names:
            specifications.append(Specification(filename=name, data=self.get_specification_data(name=name)))
        return specifications



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals
from builtins import object

import os
import shutil

from monolithe import MonolitheConfig
from monolithe.lib import Printer
from monolithe.specifications import RepositoryManager, FolderManager, SpecificationAPI


class Generator(object):

    def __init__(self, monolithe_config):
        """
        """
        self.monolithe_config = monolithe_config

    def initialize_folder_manager(self, folder):
        """
        """
        self.folder_manager = FolderManager(folder=folder, monolithe_config=self.monolithe_config)

    def retrieve_monolithe_config_from_folder(self, language="python"):
        """
        """
        parser = self.folder_manager.get_monolithe_config()
        self.monolithe_config = MonolitheConfig()
        self.monolithe_config.set_config(parser)
        self.monolithe_config.language = language
        self.folder_manager.monolithe_config = self.monolithe_config
        return self.monolithe_config

    def generate_from_folder(self):
        """
        """
        specification_info = []

        Printer.log("retrieving specifications from folder \"%s\"" % (self.folder_manager.folder))
        api_info = self.folder_manager.get_api_info()
        specifications = self.folder_manager.get_all_specifications()
        self._resolve_parent_apis(specifications)
        specification_info.append({"specifications": specifications, "api": api_info})
        Printer.log("%d specifications retrieved from folder \"%s\" (api version: %s)" % (len(specifications), self.folder_manager.folder, api_info["version"]))

        self.generate(specification_info=specification_info)

    def initialize_repository_manager(self, api_url, login_or_token, password, organization, repository, repository_path):
        """
        """
        self.repository_manager = RepositoryManager(monolithe_config=self.monolithe_config,
                                                    api_url=api_url,
                                                    login_or_token=login_or_token,
                                                    password=password,
                                                    organization=organization,
                                                    repository=repository,
                                                    repository_path=repository_path)

    def retrieve_monolithe_config_from_repo(self, branch, language):
        """
        """
        parser = self.repository_manager.get_monolithe_config(branch=branch)
        self.monolithe_config = MonolitheConfig()
        self.monolithe_config.set_config(parser)
        self.monolithe_config.language = language
        self.repository_manager.monolithe_config = self.monolithe_config
        return self.monolithe_config

    def generate_from_repo(self, branches):
        """
        """
        specification_info = []

        for branch in branches:
            Printer.log("retrieving specifications from github \"%s/%s%s@%s\"" % (self.repository_manager.organization.lower(), self.repository_manager.repository.lower(), self.repository_manager.repository_path, branch))
            api_info = self.repository_manager.get_api_info(branch=branch)
            specifications = self.repository_manager.get_all_specifications(branch=branch)
            self._resolve_parent_apis(specifications)
            specification_info.append({"specifications": specifications, "api": api_info})
            Printer.log("%d specifications retrieved from branch \"%s\" (api version: %s)" % (len(specifications), branch, api_info["version"]))

        self.generate(specification_info=specification_info)

    def generate(self, specification_info):
        """
        """
        pass

    def install_user_vanilla(self, user_vanilla_path, output_path, multi_lang=True):
        """
        """
        if not user_vanilla_path or not len(user_vanilla_path):
            return

        if multi_lang:
            user_vanilla_path = os.path.join(user_vanilla_path, self.monolithe_config.language)
        else:
            user_vanilla_path = os.path.join(user_vanilla_path)

        if not os.path.exists(user_vanilla_path):
            Printer.warn("Could not find user vanilla folder at path %s. Ignoring" % user_vanilla_path)
            return

        for item in os.listdir(user_vanilla_path):
            s = os.path.join(user_vanilla_path, item)
            d = os.path.join(output_path, item)
            if os.path.isdir(s):
                shutil.copytree(s, d, False, None)
            else:
                shutil.copy2(s, d)

    # Utilities
    def _resolve_parent_apis(self, specifications):
        """
        """

        # certainly not the best algo ever... but I need to get somthing done :)
        for specification_rest_name, specification in specifications.items():

            for rest_name, remote_spec in specifications.items():

                for related_child_api in remote_spec.child_apis:

                    if related_child_api.rest_name == specification.rest_name:

                        parent_api = SpecificationAPI(specification=specification)
                        parent_api.rest_name = related_child_api.rest_name
                        if specification.allows_get:
                            parent_api.allows_get = True
                        if specification.allows_create:
                            parent_api.allows_create = True
                        if specification.allows_update:
                            parent_api.allows_update = True
                        if specification.allows_delete:
                            parent_api.allows_Delete = True

                        specification.parent_apis.append(parent_api)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from builtins import object
import importlib
from monolithe.lib import load_language_plugins


class Manager(object):
    """
    """

    def __init__(self, monolithe_config, target_name):
        """
        """
        self.monolithe_config = monolithe_config
        self.target_name = target_name

    def get_managed_class(self):
        """
        """
        language = self.monolithe_config.language
        found, klass = load_language_plugins(language, self.target_name)

        if found:
            return klass

        try:
            module = importlib.import_module('.lang.%s' % language, package="monolithe.generators")
        except:
            raise Exception('Unsupported language %s.' % language)

        if not hasattr(module, self.target_name):
            return

        return getattr(module, self.target_name)
